/*
	Programma: molle.cpp
	Autore: Mattiucci Marco
	Scopo: Simulare il comportamento di un sistema massa-molla.
	Versione:1.3
	Data Ultima revisione: 21 gennaio 95
*/

//File include:
#include <stdio.h>
#include <stdlib.h>
#include <graphics.h>
#include <math.h>
#include <conio.h>
#include <dos.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <alloc.h>

//Codici ASCII dei caratteri di controllo dei tasti:
#define F1	59
#define F2	60
#define F3	61
#define F4	62
#define ESC	27
#define DEL	8
#define UP	72
#define DOWN	80
#define RETURN	13

//Costanti booleane:
#define TRUE	1
#define FALSE	0

//Costante fisica (accelerazione di gravit):
#define G	9.8

//Codici ASCII dei caratteri di controllo dei tasti:
#define POSIZIONE	'1'
#define VELOCITA	'2'
#define FASE		'3'
#define ORBITA		'4'

//Soglia di innesco dell'adattamento della scala dei tempi nella
// simulazione del movimento dei sistemi massa-molla.
#define SOGLIA		0.1



// Funzione per la gestione dell'input di numeri reali nella finestra
// grafica.
int readfloat(int x, int y, float *n) {
  #define NUM_DIGIT	25	//Numero massimo di digit per un numero
  #define PIX_FOR_CHAR	8	//Numero di pixel orizzontali per un digit
  char c;		//Carattere singolo di input.
  char num[NUM_DIGIT];	//stringa in input di numeri.
  int i;	    	//Contatore.
  int x1=x;		//Variabile di comodo.
  int punto_ok=FALSE;	//flag per l'immissione del punto.
  long float n1;
  setcolor(WHITE);
  outtextxy(x,y,"_");
  strcpy(num,"");
  do {
    do {// lampeggio cursore:
      setcolor(BLACK);outtextxy(x,y,"_");
      delay(60);
      setcolor(WHITE);outtextxy(x,y,"_");
      delay(60);
    } while (!kbhit());
    c=getch(); //Immissione singola cifra:
    if (isdigit(c) || (c=='-') || ((c=='.') && (!punto_ok))) {
      char str[3];				   // Un carattere  ammesso
      sprintf(str,"%c",c);			   // se  un numero oppure
      setcolor(BLACK);outtextxy(x,y,"_");	   // un punto.
      setcolor(WHITE);outtextxy(x,y,str);
      x+=PIX_FOR_CHAR;
      outtextxy(x,y,"_");
      strcat(num,str);
      if (c=='.') punto_ok=TRUE;
    } else if ((c==DEL) && (x>x1)) {		//Se il carattere  di controllo
	int new_lung=strlen(num)-1;		//per delete allora si procede
	char tmp[NUM_DIGIT];			//ad eliminare l'ultimo numero
	setcolor(BLACK);outtextxy(x,y,"_");	//immesso.
	x-=PIX_FOR_CHAR;
	setfillstyle(SOLID_FILL,BLACK);
	bar(x,y,x+PIX_FOR_CHAR,y+10);
	strncpy(tmp,num,new_lung);
	tmp[new_lung]='\0';
	if (num[new_lung]=='.') punto_ok=FALSE;
	strcpy(num,tmp);
    } else if (c!=RETURN) { //per caratteri non ammessi c' un bip di avviso:
	sound(100); delay(100); nosound();
    }//if.
  } while ((c!=ESC) && (c!=RETURN));
  if (c==ESC) return FALSE;
  if (!punto_ok) strcat(num,".");
  for(i=strlen(num); i<=NUM_DIGIT-1; i++) strcat(num,"0");
  sscanf(num,"%lf",&n1);
  *n=(float)n1;
  return TRUE;
}//readfloat.



//*********************************************************
// Classe schermo, utile per le finestre a scomparsa.
// Questa classe permette di salvare l'intero contenuto della
// pagina grafica sotto forma di oggetto e poi di riutilizzarla.
//*****************************************************************
typedef class screen_class {
private:
  void far *buf[4];
  int maxx,maxy;
public:
  screen_class();
  void save();
  void restore();
} screen_obj;
//------------------------------------------------
screen_class::screen_class() {
  maxx=getmaxx();
  maxy=getmaxy();
}//End of constructor.
//------------------------------------------
void screen_class::save()  {
   unsigned size;
   int ystart=0, yend, yincr, block;
   yincr = (maxy+1) / 4;
   yend = yincr;
   size = imagesize(0, ystart, maxx, yend);
   for (block=0; block<=3; block++)   {
      if ((buf[block] = farmalloc(size)) == NULL)   {
	 closegraph();
	 printf("Error: not enough heap space in save_screen().\n");
	 exit(1);
      }//if.
      getimage(0, ystart, maxx, yend, buf[block]);
      ystart = yend + 1;
      yend += yincr + 1;
   }//for.
}//save_screen.
//-----------------------------------------------
void screen_class::restore()  {
   int ystart=0, yend, yincr, block;
   yincr = (maxy+1) / 4;
   yend = yincr;
   for (block=0; block<=3; block++)   {
      putimage(0, ystart, buf[block], COPY_PUT);
      farfree(buf[block]);
      ystart = yend + 1;
      yend += yincr + 1;
   }//for.
}//restore_screen.
//*****************************************************************



/**************************************************************/
// Definizione della classe corrispondente all'oggetto massa
// rappresentato nell'animazione. Questo prototipo di oggetto
// permette di muovere verticalmente da una posizione ad un'altra
// dello schermo un blocco rettangolare di dimensione fissa.
/**************************************************************/
typedef class m_class_name {
private:
  int dim;  // Dimensione del blocco rettangolare.
  int x0,y0;// Posizione nello schermo in pixel.
public:
  m_class_name(int,int,int);  // Costruttore del blocco.
  void move(int,int);	      // Funzione movimento.
} m_class; //End Of Class m_class_name.
//--------------------------------------------
  m_class_name::m_class_name(int x, int y, int d) {
    int i; 		// Contatore.
    x0=x; y0=y; dim=d;	// Settaggio dei parametri.
    setfillstyle(9,7);
    bar(x0-dim, y0, x0+dim, y0+dim);
  }//End Of Constructor.
//---------------------------------------------
  void m_class_name::move(int x, int y) {
    if (y<y0) {
      setfillstyle(0,0);
      bar(x0-dim, y+dim, x0+dim, y0+dim);
      setfillstyle(9,7);
      bar(x0-dim, y, x0+dim, y0);
    } else {
      setfillstyle(0,0);
      bar(x0-dim, y0, x0+dim, y);
      setfillstyle(9,7);
      bar(x0-dim, y0+dim, x0+dim, y+dim);
    }//if.
    x0 = x;    y0 = y;
  }//End Of Move Proc.
/************************************************************************/





/***********************************************************************/
// Definizione del prototipo di molla da rappresentare sullo schermo.
// Questa definizione di molla permette di spostare l'intera molla oppure
// di accorciarla o allungarla. La molla  uno zig-zag di larghezza
// calcolata a seconda dell'allungamento.
/***********************************************************************/
typedef class k_class_name {
private:
  int x0,y0;  // Posizione in alto del "gancio" della molla.
  int step;   // Passo di incremento nel disegno della molla:
  int w;      // Larghezza della molla.
  int b;      // Lunghezza dei singoli segmenti.
  float l;    // Lunghezza della molla.
  int inc;    // Piega finale della molla ( utile nel caso parallelo ).
public:
  k_class_name(int,int,float,int); // Costruttore.
  void new_length(int,int,int);	   // Funzione per cambiare posizione e lunghezza.
} k_class;//End Of Class k_class_name.
//---------------------------------------------------
  k_class_name::k_class_name(int x, int y, float l1, int inc1) {
    x0 = x;
    y0 = y;
    l = l1;
    b = 35;
    inc = inc1;
    step = (l-2*b)/6;
    w = (int)(sqrt((double)(b)*(double)(b)-(double)(step)*(double)(step)));
    moveto(x0,y0);
    setcolor(WHITE);
    setlinestyle(SOLID_LINE,1,1);
    lineto(x0, y0+b);
    lineto(x0-w, y0+b+step);
    lineto(x0+w, y0+b+2*step);
    lineto(x0-w, y0+b+3*step);
    lineto(x0+w, y0+b+4*step);
    lineto(x0-w, y0+b+5*step);
    lineto(x0, y0+b+6*step);
    lineto(x0, y0+2*b+6*step);
    lineto(x0+inc, y0+2*b+6*step);
  }//End Of Constructor.
//---------------------------------------------------
  void k_class_name::new_length(int x, int y, int l1) {
    moveto(x0,y0);
    setcolor(BLACK);
    lineto(x0, y0+b);
    lineto(x0-w, y0+b+step);
    lineto(x0+w, y0+b+2*step);
    lineto(x0-w, y0+b+3*step);
    lineto(x0+w, y0+b+4*step);
    lineto(x0-w, y0+b+5*step);
    lineto(x0, y0+b+6*step);
    lineto(x0, y0+2*b+6*step);
    lineto(x0+inc, y0+2*b+6*step);
    x0 = x;
    y0 = y;
    l = l1;
    step = (l-2*b)/6;
    w = (int)(sqrt((double)(b)*(double)(b)-(double)(step)*(double)(step)));
    moveto(x0,y0);
    setcolor(WHITE);
    lineto(x0, y0+b);
    lineto(x0-w, y0+b+step);
    lineto(x0+w, y0+b+2*step);
    lineto(x0-w, y0+b+3*step);
    lineto(x0+w, y0+b+4*step);
    lineto(x0-w, y0+b+5*step);
    lineto(x0, y0+b+6*step);
    lineto(x0, y0+2*b+6*step);
    lineto(x0+inc, y0+2*b+6*step);
  }//End Of New Length Proc.
/*****************************************************************/





/*****************************************************************/
// Classe per la gestione delle finestre grafiche.
// Questa definizione non tratta finestre a scomparsa che lasciano
// inalterato il contenuto sotto; per ottenere ci  necessario
// operare una gestione a parte.
/*****************************************************************/
typedef class open_win {
protected:
  int x0, y0, x1, y1; // Posizione dell'angolo alto sinistro e di quello
		      // in basso a destra della finestra.
  char *name;	      // Intestazione della finestra.
  int width;	      // Larghezza della striscia di intestazione.
public:
  open_win(int,int,int,int,char*);  // Costruttore.
  void recall_win();		    // Ravviva il colore dell'intestazione.
  void parq_win();		    // Opacizza il colore dell'intestazione.
  void pos(int,int);		    // Posiziona la scrittura e lettura in x,y.*/
  void textxy(int,int,char*);  	    // Scrive una stringa in x,y.
  void clear();			    // Pulisce la finestra.
  ~open_win();			    // Distruttore.
} my_win;//End Of Class Open Window.
//---------------------------------
  open_win::open_win(int xleft, int yleft, int xright, int yright, char* name1) {
    x0 = xleft;
    y0 = yleft;
    x1 = xright;
    y1 = yright;
    width = 15;
    name = name1;
    setfillstyle(0,0);
    bar(x0, y0, x1, y1);
    setfillstyle(9,RED);
    bar(x0, y0, x1, y0+width);
    setfillstyle(SOLID_FILL,GREEN);
    bar(x0+5, y0+2, x0+15, y0+width-2);
    setcolor(7);
    setlinestyle(SOLID_LINE,1,3);
    rectangle(x0, y0, x1, y1);
    setlinestyle(SOLID_LINE,1,1);
    line(x0, y0+width, x1, y0+width);
    setcolor(WHITE);
    outtextxy((x1+x0)/2-strlen(name)*4, y0+4, name);
  }//End Of Constructor.
//-----------------------------------------------------
  void open_win::parq_win() {
    setfillstyle(9,7);
    bar(x0, y0, x1, y0+width);
    setcolor(WHITE);
    outtextxy((x1+x0)/2-strlen(name)*4, y0+4, name);
  }//End Of Proc parq_win.
//-----------------------------------------------------
  void open_win::recall_win() {
    setfillstyle(9,RED);
    bar(x0, y0, x1, y0+width);
    setfillstyle(SOLID_FILL,GREEN);
    bar(x0+5, y0+2, x0+15, y0+width-2);
    setcolor(7);
    setlinestyle(SOLID_LINE,1,3);
    rectangle(x0, y0, x1, y1);
    setlinestyle(SOLID_LINE,1,1);
    line(x0, y0+width, x1, y0+width);
    setcolor(WHITE);
    outtextxy((x1+x0)/2-strlen(name)*4, y0+4, name);
  }//End Of Proc. recall_win
//---------------------------------------------------
  void open_win::pos(int x, int y) {
    int riga=(y0+width+y)/14;
    int colonna=(x0+x)/7;
    gotoxy(colonna,riga);
  }//End Of Proc textxy.
//---------------------------------------------------
  void open_win::textxy(int x, int y, char* txt) {
    outtextxy(x0+x, y0+width+y, txt);
  }//End Of Proc textxy.
//---------------------------------------------------
  void open_win::clear() {
    setfillstyle(0,0);
    bar(x0+2, y0+width+2, x1-2, y1-2);
  }//End Of Proc Clear.
//---------------------------------------------------
  open_win::~open_win() {
    setfillstyle(0,0);
    bar(x0-1, y0-1, x1+1, y1+1);
  }//End Of Destructor.
/*********************************************************************/





/*********************************************************************/
// Oggetto freccia utile nei men. Viene utilizzato nella scelta dei
// parametri. La freccia pu essere resa di colore diverso a seconda
// di ci che accade e pu essere spostata sullo schermo.
/*********************************************************************/
typedef class arrow_class {
private:
  int x0,y0;	// Posizione della freccia.
  int poly[6];	// Vettore contenente le posizioni degli spigoli della freccia.
  int dim;	// Dimensione caratteristica della freccia.
public:
  arrow_class(int,int);	// Costruttore.
  void move(int,int);	// Movimento.
  void deactive();	// Colore scuro.
  void active();	// Colore rosso.
} arrow;//End Of Class Arrow
//------------------------------
  arrow_class::arrow_class(int x, int y) {
    dim = 10;
    x0 = x;
    y0 = y;
    poly[0] = x0-dim; 	poly[1] = y0;
    poly[2] = x0+dim/2;	poly[3] = y0-dim/2;
    poly[4] = poly[2];  poly[5] = y0+dim/2;
    setfillstyle(SOLID_FILL,RED);
    setcolor(WHITE);
    fillpoly(3,poly);
  }//End Of constructor.
//---------------------------------------
  void arrow_class::move(int x, int y) {
    setfillstyle(SOLID_FILL,BLACK);
    bar(x0-dim, y0-dim/2, x0+dim/2, y0+dim/2);
    x0 = x;
    y0 = y;
    poly[0] = x0-dim; 	poly[1] = y0;
    poly[2] = x0+dim/2;	poly[3] = y0-dim/2;
    poly[4] = poly[2];  poly[5] = y0+dim/2;
    setfillstyle(SOLID_FILL,RED);
    setcolor(WHITE);
    fillpoly(3,poly);
  }//End Of Proc move
//------------------------------------------
  void arrow_class::deactive() {
    setfillstyle(SOLID_FILL,BLACK);
    bar(x0-dim, y0-dim/2, x0+dim/2, y0+dim/2);
    setcolor(WHITE);
    drawpoly(3,poly);
  }//End Of Proc. deactive
//--------------------------------------------
  void arrow_class::active() {
    setfillstyle(SOLID_FILL,RED);
    setcolor(WHITE);
    fillpoly(3,poly);
  }//End of Proc active.
/*********************************************************/



// PROCEDURA PER LA MODIFICA DA UTENTE DEI PARAMETRI DEL SISTEMA:
void parameter(	float *m,
		float *k1,
		float *k2,
		float *Dl,
		float *Dl1,
		float *Xu,
		float *Vu,
		float *Tu,
		float *mu) {
  int x0=200, y0=200, x1=550, y1=400;
  screen_obj screen;	// La finestra parametri
  my_win *qwin;		//  una finestra a scomparsa, allora  necessario
  arrow *ar;		// salvare il contenuto che copre prima di
  int p=y0+30, a1;	// disegnarla, al fine di poter ripristinare il
  int inc=16;		// contenuto dopo.
  int num_opz=9;
  screen.save();
  qwin = new my_win(x0, y0, x1, y1, "PARAMETRI MODIFICABILI");
  qwin->pos(10,5);printf("k1   = %f [N/m]",*k1);
  qwin->pos(10,20);printf("k2   = %f [N/m]",*k2);
  qwin->pos(10,35);printf("m    = %f [kg]",*m);
  qwin->pos(10,45);printf("Xs0 = %f [m]",*Dl);
  qwin->pos(10,55);printf("Xp0 = %f [m]",*Dl1);
  qwin->pos(10,65);printf("Xu   = %f [m]",*Xu);
  qwin->pos(10,80);printf("Vu   = %f [m/s]",*Vu);
  qwin->pos(10,95);printf("Tu   = %f [s]",*Tu);
  qwin->pos(10,110);printf("%c   = %f [N*s/m]",230,*mu);
  setcolor(WHITE);
  outtextxy(10,5,"[UP/DOWN] per scegliere.  [RETURN] per selezionare.  [ESC] per finire.");
  ar = new arrow(x1-80, p);
  do {
      a1 =  getch();
      if (a1==0) {
	a1 = getch();
	if (a1==UP) {
	  if (p==y0+30) {
	    sound(100);delay(100); nosound();
	  } else {
	    p -= inc; ar->move(x1-80, p);
	  }//if.
	} else if (a1==DOWN) {
	  if (p>=y0+30+inc*(num_opz-1)) {
	    sound(100); delay(100); nosound();
	  } else {
	    p += inc; ar->move(x1-80, p);
	  }//if.
	}//if.
      } else if (a1==RETURN) {
	ar->deactive();
	int choise=(p-30-y0)/inc+1;
	switch (choise) {
	  case 1: {
	    qwin->pos(50,130);printf("Nuova k1: . . . . . . . . ");
	    readfloat(370,372,k1);
	    qwin->pos(50,130);printf("                               ");
	    qwin->pos(10,5);printf("k1   = %f [N/m]",*k1);
	    break;
	  }//1
	  case 2: {
	    qwin->pos(50,130);printf("Nuova k2: . . . . . . . . ");
	    readfloat(370,372,k2);
	    qwin->pos(50,130);printf("                               ");
	    qwin->pos(10,20);printf("k2   = %f [N/m]",*k2);
	    break;
	  }//2
	  case 3: {
	    qwin->pos(50,130);printf("Nuova m: . . . . . . . . ");
	    readfloat(370,372,m);
	    qwin->pos(50,130);printf("                               ");
	    qwin->pos(10,35);printf("m    = %f [kg]",*m);
	    break;
	  }//3
	  case 4: {
	    qwin->pos(50,130);printf("Nuova X0s: . . . . . . . . ");
	    readfloat(370,372,Dl);
	    qwin->pos(50,130);printf("                               ");
	    qwin->pos(10,45);printf("Xs0 = %f [m]",*Dl);
	    break;
	  }//4
	  case 5: {
	    qwin->pos(50,130);printf("Nuova X0p: . . . . . . . . ");
	    readfloat(370,372,Dl1);
	    qwin->pos(50,130);printf("                               ");
	    qwin->pos(10,55);printf("Xp0 = %f [m]",*Dl1);
	    break;
	  }//5
	  case 6: {
	    qwin->pos(50,130);printf("Nuova Xu: . . . . . . . . ");
	    readfloat(370,372,Xu);
	    qwin->pos(50,130);printf("                               ");
	    qwin->pos(10,65);printf("Xu   = %f [m]",*Xu);
	    break;
	  }//6
	  case 7: {
	    qwin->pos(50,130);printf("Nuova Vu: . . . . . . . . ");
	    readfloat(370,372,Vu);
	    qwin->pos(50,130);printf("                               ");
	    qwin->pos(10,80);printf("Vu   = %f [m/s]",*Vu);
	    break;
	  }//7
	  case 8: {
	    qwin->pos(50,130);printf("Nuovo Tu: . . . . . . . . ");
	    readfloat(370,372,Tu);
	    qwin->pos(50,130);printf("                               ");
	    qwin->pos(10,95);printf("Tu   = %f [s]",*Tu);
	    break;
	  }//8
	  case 9: {
	    qwin->pos(50,130);printf("Nuovo %c: . . . . . . . . ",230);
	    readfloat(370,372,mu);
	    qwin->pos(50,130);printf("                               ");
	    qwin->pos(10,110);printf("%c   = %f [N*s/m]",230,*mu);
	    break;
	  }//9
	}//Switch
	ar->active();
      }//if.
  } while (a1!=ESC);
  delete qwin;
  delete ar;
  screen.restore();
//  }//if.
}




//     	PROCEDURA CHE RILEVA LA SCELTA NEL MENU' PRINCIPALE DELL'UTENTE:
int command() {
  int a1, a2;
  setfillstyle(BLUE,SOLID_FILL);
  bar(1,1,640,16);
  setcolor(WHITE);
  outtextxy(10,5,"F1");
  setcolor(7);
  outtextxy(30,5,"AIUTO.");
  setcolor(WHITE);
  outtextxy(100,5,"F2");
  setcolor(7);
  outtextxy(120,5,"PARAMETRI.");
  setcolor(WHITE);
  outtextxy(230,5,"F3");
  setcolor(7);
  outtextxy(250,5,"SIMULAZIONE.");
  setcolor(WHITE);
  outtextxy(380,5,"F4");
  setcolor(7);
  outtextxy(400,5,"GRAFICI.");
  setcolor(WHITE);
  outtextxy(490,5,"ESC");
  setcolor(7);
  outtextxy(520,5,"USCITA.");
  do
    if ((a1=getch())==0) {
      a2=getch();
      if ((a2<=F4) && (a2>=F1)) {
	bar(1,1,640,16);
	return a2;
      }
    }
  while (a1!=ESC);
  return ESC;
}



// Procedura che disegna il "muro" di appoggio delle molle:
void ground(int x, int y) {
  int dim=20;
  setfillstyle(5,7);
  bar(x-dim, y-dim, x+dim, y);
  setlinestyle(SOLID_LINE,1,3);
  setcolor(WHITE);
  line(x-dim, y, x+dim, y);
}


// Funzione di gestione degli overflow e underflow matematici.
//  La presente funzione viene richiamata in automatico dal sistema nel
//  momento in cui occorre un errore del tipo sopra citato. Il programma
//  quindi non si ferma in tali eventualit e non solo questo, infatti
//  la funzione che ha causato l'errore se  un esponenziale ritorna 0.
int matherr (struct exception *a)
{
  if (a->type == OVERFLOW)
    if (!strcmp(a->name,"exp")) {
      a->retval = 0.;
      return 1;
  }//if.
  if (a->type == UNDERFLOW)
    if (!strcmp(a->name,"exp")) {
      a->retval = 0.;
      return 1;
  }//if.
  return 0;
}




/*************************************************************/
/************* PROGRAMMA PRINCIPALE **************************/
/*************************************************************/
void main() {
  //Variabili per la grafica di sistema:
  int gdriver = DETECT, gmode, errorcode;

  //Variabili di comodo:
  int i;
  int no_exit;

  //Variabili oggetto per le finestre utilizzate permanentemente:
  my_win *simul_win; // Finestra di simulazione e grafica.
  my_win *data_win;  // Finestra dati.

  //Variabili oggetto per la simulazione:
  k_class *Ks_1;  // Prima molla in serie.
  k_class *Ks_2;  // Seconda molla in serie.
  m_class *Ms;    // Massa nel caso serie.
  k_class *Kp_1;  // Prima molla nel caso parallelo.
  k_class *Kp_2;  // Seconda molla nel caso parallelo.
  m_class *Mp;    // Massa nel caso parallelo.

  //Variabili di calcolo per la simulazione:
  int Dl1si;  	// Analogo discreto di Dl1st.
  int Dl2si;	// Analogo discreto di Dl2st.
  int Dlpi;	// Analogo discreto di Dlpt
  float Dl1st;	// Lunghezza della prima molla in serie funzione del tempo.
  float Dl2st;  // Lunghezza della seconda molla in serie funzione del tempo.
  float Dlpt;	// Lunghezza delle molle in parallelo funzione del tempo.
  float Dl0s=0.;	//Condizione iniziale di default per la pos, caso serie.
  float Dl0p=0.;	//Condizione iniziale di default per la pos, caso parallelo.
  float Xunit=0.6;	//Unit di default dello spostamento 0.6 m.
  float Vunit=2.;	//Unit di default della velocit 2 m/s.
  float Tunit=1.;	//Unit di default del tempo 1 s.
  float m=1.;		//Massa di default 1 kg.
  float k1=10.;		//Prima costante elastica di default 10 N/m.
  float k2=20.;		//Seconda costante elastica di deafult 20 N/m.
  float mu=0.;  	//Coefficiente di attrito di deafult 0 N*s/m.

  //Parametri caratteristici grafici delle molle:
  int xs0=70;	//Posizione in ascissa delle molle in serie.
  int ys10=60;  //Posizione in ordinata del gancio (di sopra) della prima molla.
  int l0=100;  //Lunghezza di riposo della prima molla (forza nulla).
  int ys20=ys10+l0; //Posizione in ordinata del gancio (di sopra) della seconda molla.
  int xp10=300;  //Posizione in ascissa della prima molla in parallelo.
  int xp20=340;  //Posizione in ascissa della prima molla in parallelo.
  int yp0=ys20; //Posizione in ordinata del gancio delle molle in parallelo.

  //Inizializzazione della grafica:
  initgraph(&gdriver, &gmode, "");
  errorcode = graphresult();
  if (errorcode != grOk) {
    printf("Graphics error: %s\n", grapherrormsg(errorcode));
    printf("Press any key to halt:");
    getch();
    exit(1);
  }//if.

  //Presentazione del programma:
  setfillstyle(0,0);
  setlinestyle(SOLID_LINE,1,3);
  for(i=1;i<=25;i++) {  //Costruzione del rettangolo che cresce:
    setcolor(0);
    rectangle(320-5*(i-1),240-4*(i-1),320+5*(i-1),240+4*(i-1));
    setcolor(GREEN);
    rectangle(320-5*i,240-4*i,320+5*i,240+4*i);
    delay(20);
  }
  randomize();  //Riempimento del rettangolo:
  for(i=0;i<=5000;i++)
    putpixel(197+random(247),142+random(197),RED);
  for(i=0;i<=5000;i++)
    putpixel(197+random(247),142+random(197),random(9));
  setcolor(WHITE);
  outtextxy(210,150,"MOLLE 1.3  -   Gennaio 1995");
  outtextxy(210,170,"Autore: Mattiucci Marco");
  outtextxy(210,190,"Matricola: 210337");
  outtextxy(210,210,"Ing. Elettronica Ancona");
  outtextxy(210,230,"Esame: Meccanica Razionale");
  outtextxy(210,250,"Prof.: G. Frosali");
  setcolor(YELLOW);
  outtextxy(210,290,"  Premere un tasto...");
  do {	//Riempimento dello sfondo:
    int x=random(640), y=random(480);
    if (!( (x>193) && (x<448) && (y>138) && (y<343) ))
	putpixel(x,y,random(MAXCOLORS));
  } while (!kbhit());
  getch();
  clearviewport();

  //Inizializzazione finestre di lavoro:
  simul_win = new my_win(1, 17, 450, 480, "MOLLE 1.3  -  SIMULAZIONE E GRAFICI");
  simul_win->parq_win();
  data_win = new my_win(457, 17, 637, 480,"MOLLE 1.3 - DATI");
  data_win->parq_win();

  //Attivazione men comandi:
  no_exit = TRUE;
  do { //LOOP PRINCIPALE DEI COMANDI:
    switch (command()) {

      case F1:  {// RICHIAMO FILE DI AIUTO molle.hlp
	delete simul_win;
	delete data_win;
	restorecrtmode();
	system("type molle.hlp|more");
	getch();
	setgraphmode(getgraphmode());
	simul_win = new my_win(1, 17, 450, 480, "MOLLE 1.3  -  SIMULAZIONE E GRAFICI");
	simul_win->parq_win();
	data_win = new my_win(457, 17, 637, 480,"MOLLE 1.3 - DATI");
	data_win->parq_win();
	break;
      }//F1

      case F2: parameter(&m, &k1, &k2, &Dl0s, &Dl0p, &Xunit, &Vunit, &Tunit, &mu); break;

      case F3: { //INIZIO SIMULAZIONE:

	//Inizializzazione di variabili locali:
	float ks=(k1*k2)/(k1+k2); //Costante equivalente della serie delle molle.
	float kp=k1+k2;	          //Costante equivalente del parallelo delle molle.
	float ws=sqrt(ks/m);  //Frequenza propria del sistema, caso serie.
	float wp=sqrt(kp/m);  //Frequenza propria del sistema, caso parallelo.
	float eps=mu/(2*m);   //Inverso della costante di tempo del sistema.
	float alfas=sqrt(fabs(eps*eps-ws*ws)); //Frequenza, caso serie, viscosit non nulla.
	float alfap=sqrt(fabs(eps*eps-wp*wp)); //Frequenza, caso paral., viscosit non nulla.
	float fases=atan(eps/alfas); //Fase iniziale, caso serie.
	float fasep=atan(eps/alfap); //Fase iniziale, caso parallelo.
	float Cs=sqrt(fabs(ws*ws/(ws*ws-eps*eps))); //Var. di comodo.
	float Cp=sqrt(fabs(wp*wp/(wp*wp-eps*eps))); //Var. di comodo.
	float C1s=1.+eps/alfas; //Var. di comodo.
	float C2s=1.-eps/alfas; //Var. di comodo.
	float C1p=1.+eps/alfap; //Var. di comodo.
	float C2p=1.-eps/alfap; //Var. di comodo.
	float Ts=2*M_PI/ws;   //Periodo della serie.
	float Tp=2*M_PI/wp;   //Periodo del parallelo.
	float As=m*G/ks;      //Soluzione asintotica, caso serie (Xp).
	float Ap=m*G/kp;      //Soluzione asintotica, caso parallelo (Xp).
	float A1=m*G/k1;      //Soluzione asintotica, solo k1.
	float A2=m*G/k2;      //Soluzione asintotica, solo k2.
	int dim_m=26;	      //Dimensione in pixel della "massa"
	float div_num=50.;    //Numero di divisioni dell'unit di tempo (discretizzazione di t)
	float pix_for_unit=20.;  //Dimensione in pixel della unit di V o X.
	float dt=Tunit/div_num;  //Incremento temporale nel loop per l'animazione.
	float t=0.;	      //Variabile tempo di simulazione.
	float kf1=k1/kp;      //Frazione proporzionale di allungamento di k2.
	float kf2=k2/kp;      //Frazione proporzionale di allungamento di k1.
	float Dl10st=kf2*Dl0s;//Allungamento di k1 per Dl0s come cond. iniz.
	float Dl20st=kf1*Dl0s;//Allungamento di k2 per Dl0s come cond. iniz.
	int Dl10si=(int)((Dl10st/Xunit)*pix_for_unit);//Allungamento iniz. in pixel k1.
	int Dl20si=(int)((Dl20st/Xunit)*pix_for_unit);//Allungamento iniz. in pixel k2
	int l10s=l0+Dl10si;  //Lunghezza iniz. in pixel k1.
	int l20s=l0+Dl20si;  //Lunghezza iniz. in pixel k2.
	int Dl0pi=(int)((Dl0p/Xunit)*pix_for_unit);//Allungamento iniz. in pixel.
	int l0p=l0+Dl0pi; //Lunghezza iniziale in pixel.
	//Massime e minime posizioni in pixel raggiunte durante l'oscillazione
	// dai sistemi massa-molla caso serie (s) e parallelo (p):
	int ysmin=(int)((As-fabs(As-Dl0s))/Xunit*pix_for_unit)+ys20+l0;
	int ysmax=(int)((As+fabs(As-Dl0s))/Xunit*pix_for_unit)+ys20+l0;
	int ypmin=(int)((Ap-fabs(Ap-Dl0p))/Xunit*pix_for_unit)+ys20+l0;
	int ypmax=(int)((Ap+fabs(Ap-Dl0p))/Xunit*pix_for_unit)+ys20+l0;
	//Flag per determinare quale soluzione usare a seconda di  e w:
	float SW1s=0., SW2s=0., SW3s=0.;
	float SW1p=0., SW2p=0., SW3p=0.;
	//Variabili per la gestione del tempo di clock del sistema:
	clock_t start;
	double tempo;
	//Variabile di comodo per I/O:
	char msg[40],msg1[10],msg2[10];

	//Scrittura dei dati calcolati sulla finestra dati:
	data_win->clear();
	gotoxy(60,4);printf("k1 = %6.3f [N/m]",k1);
	gotoxy(60,5);printf("k2 = %6.3f [N/m]",k2);
	gotoxy(60,6);printf("m  = %6.3f [kg]",m);
	gotoxy(60,7);printf("%c  = %6.3f [N*s/m]",230,mu);
	gotoxy(60,9);printf("Ts = %6.3f [s]",Ts);
	gotoxy(60,10);printf("Tp = %6.3f [s]",Tp);
	gotoxy(60,11);printf("As = %6.3f [m]",2.*fabs(As-Dl0s));
	gotoxy(60,12);printf("Ap = %6.3f [m]",2.*fabs(Ap-Dl0p));
	gotoxy(60,13);printf("X0s= %6.3f [m]",Dl0s);
	gotoxy(60,14);printf("X0p= %6.3f [m]",Dl0p);
	gotoxy(60,15);printf("Xsmin=%6.3f [m]",As-fabs(As-Dl0s));
	gotoxy(60,16);printf("Xsmax=%6.3f [m]",As+fabs(As-Dl0s));
	gotoxy(60,17);printf("Xpmin=%6.3f [m]",Ap-fabs(Ap-Dl0p));
	gotoxy(60,18);printf("Xpmax=%6.3f [m]",Ap+fabs(Ap-Dl0p));
	gotoxy(60,20);printf("Xsreg=%6.3f [m]",As);
	gotoxy(60,21);printf("Xpreg=%6.3f [m]",Ap);
	gotoxy(60,23);printf("Tu=%6.3f [s]",Tunit);

	//Riattiva e pulisce la finestra di simulazione:
	simul_win->recall_win();
	simul_win->clear();

	//Controllo rappresentabilit dell'oscillazione; se le masse
	// oscillando uscissero dallo schermo si richiede di modificare
	// le scale di rappresentazione:
	if ((ypmax>450) || (ysmax>450)) {
	  setcolor(YELLOW);
	  simul_win->textxy(60,70,"SCALA INADATTA PER LA RAPPRESENTAZIONE");
	  simul_win->textxy(60,150,"Utilizzare [F2] per eventuali modifiche...");
	  setcolor(WHITE);
	  outtextxy(10,5,"Premere un tasto qualsiasi per continuare...");
	  getch();
	  simul_win->clear();
	  simul_win->parq_win();
	  break;
	}//if.

	//Disegna supporto per le molle:
	ground(xs0,ys10);
	ground(xp10,ys10);
	ground(xp20,ys10);
	setlinestyle(SOLID_LINE,1,1);
	line(xp10,ys10,xp10,yp0);
	line(xp20,ys10,xp20,yp0);

	//Disegna assi di riferimento:
	setlinestyle(DOTTED_LINE,1,1);
	setcolor(YELLOW);
	line(140, ys20+l0, 250, ys20+l0);  //Asse di riposo per le molle.
	setlinestyle(SOLID_LINE,1,1);
	line(195, ys10, 195, 450);	//Asse x.
	line(190, 445, 195, 450);	//Freccia
	line(195, 450, 200, 445);	//Freccia
	outtextxy(195, 455, "X");
	outtextxy(200, ys20+l0-10, "0");
	setcolor(7);
	for(i=20; i<=180; i+=20) {		//Tacche sull'asse x:
	  line(190, i+ys20+l0, 200, i+ys20+l0);
	  line(190, -i+ys20+l0, 200, -i+ys20+l0);
	}//for.
	line(20, 440, 20, 440+(int)pix_for_unit);  //Unit di esempio
	line(15, 440, 25, 440);
	line(15, 440+(int)pix_for_unit, 25, 440+(int)pix_for_unit);
	sprintf(msg,"Unit: %6.3f [m]",Xunit);
	outtextxy(30,440+(int)(pix_for_unit/2),msg);
	setcolor(RED);
	line(140, ysmin, 160, ysmin); //estremo alto oscillazione serie
	line(140, ysmax, 160, ysmax); //estremo basso oscillazione serie
	line(150, ysmin, 150, ysmax);
	circle(150, ysmin, 4);
	outtextxy(145, ysmin-20, "Xsmin");
	circle(150, ysmax, 4);
	outtextxy(145, ysmax+15, "Xsmax");
	setcolor(WHITE);
	outtextxy(160, (ysmin+ysmax)/2, "As");
	setcolor(RED);
	line(230, ypmin, 250, ypmin); //estremo alto oscillazione parallelo
	line(230, ypmax, 250, ypmax); //estremo basso oscillazione parallelo
	line(240, ypmin, 240, ypmax);
	circle(240, ypmin, 4);
	outtextxy(215, ypmin-20, "Xpmin");
	circle(240, ypmax, 4);
	outtextxy(215, ypmax+15, "Xpmax");
	setcolor(WHITE);
	outtextxy(220, (ypmin+ypmax)/2, "Ap");

	//Crea oggetti, le molle e le masse:
	Ks_1 = new k_class (xs0, ys10, l10s, 0);
	Ks_2 = new k_class (xs0, ys10+l10s, l20s, 0);
	Ms = new m_class (xs0, ys10+l10s+l20s, dim_m);
	Kp_1 = new k_class (xp10, yp0, l0p, 0);
	Kp_2 = new k_class (xp20, yp0, l0p, 0);
	Mp = new m_class ((xp10+xp20)/2, yp0+l0p, dim_m);
	setcolor(MAGENTA);
	outtextxy(230,445,"TEMPO REALE:          [s]");
	outtextxy(230,466,"SIMULAZIONE:          [s]");
	setlinestyle(SOLID_LINE,1,3);
	setcolor(7);
	rectangle(220,437,450,480);
	line(220,458,450,458);
	setlinestyle(SOLID_LINE,1,1);

	//Scrive sulla striscia men:
	setcolor(WHITE);
	outtextxy(10,5,"Premere un tasto qualsiasi per inizio animazione...");
	getch();
	setfillstyle(SOLID_FILL,BLUE);
	bar(1,1,640,16);
	outtextxy(10,5,"[ESC] per smettere...");

	//Si confrontano eps, wp ed ws al fine vedere quale soluzione
	// utilizzare della equazione differenziale che regge il sistema:
	if (eps==ws) SW2s=1.;
	else if (eps>ws) SW3s=1.;
	     else SW1s=1.;
	if (eps==wp) SW2p=1.;
	else if (eps>wp) SW3p=1.;
	     else SW1p=1.;

	//Si fa partire il timer hardware della macchina:
	start=clock();
	strcpy(msg1,"000.00");
	strcpy(msg2,"000.00");

	//LOOP DI ANIMAZIONE:
	do do {

	  //Calcolo dell'allungamento all'istante t in [m]:+
	  double E=exp(-eps*t);
	  double E1s=exp(alfas*t);
	  double E2s=exp(-alfas*t);
	  double E1p=exp(alfap*t);
	  double E2p=exp(-alfap*t);
	  Dl1st = A1 + (Dl10st-A1)*
		( SW1s*E*Cs*cos(alfas*t-fases) +
		  SW2s*E*(1.+eps*t) +
		  SW3s*0.5*E*( C1s*E1s + C2s*E2s ));
	  Dl2st = A2 + (Dl20st-A2)*
		( SW1s*E*Cs*cos(alfas*t-fases) +
		  SW2s*E*(1.+eps*t) +
		  SW3s*0.5*E*( C1s*E1s + C2s*E2s ));
	  Dlpt  = Ap + (Dl0p-Ap)*
		( SW1p*E*Cp*cos(alfap*t-fasep) +
		  SW2p*E*(1.+eps*t) +
		  SW3p*0.5*E*( C1p*E1p + C2p*E2p ));

	  //Calcolo dell'allungamento all'istante t in pixel:
	  Dl1si = (Dl1st/Xunit*pix_for_unit);
	  Dl2si = (Dl2st/Xunit*pix_for_unit);
	  Dlpi  = (Dlpt/Xunit*pix_for_unit);

	  //Spostamento degli oggetti:
	  Ks_1->new_length(xs0, ys10, l0+Dl1si);
	  Ks_2->new_length(xs0, ys10+l0+Dl1si, l0+Dl2si);
	  Ms->move(xs0, ys10+2*l0+Dl2si+Dl1si);
	  Kp_1->new_length(xp10, yp0, l0+Dlpi);
	  Kp_2->new_length(xp20, yp0, l0+Dlpi);
	  Mp->move((xp10+xp20)/2, yp0+l0+Dlpi);

	  //Salto a istante successivo:
	  t += dt;

	  //Mostra il tempo di simulazione e quello di orologio:
	  setcolor(BLACK);
	  outtextxy(340,445,msg1);
	  outtextxy(340,466,msg2);
	  setcolor(YELLOW);
	  sprintf(msg1,"%6.2f",t);
	  outtextxy(340,445,msg1);
	  sprintf(msg2,"%6.2f",(tempo=(clock()-start)/CLK_TCK));
	  outtextxy(340,466,msg2);

	} while (!kbhit()); while (getch()!=ESC); //FINE LOOP ANIMAZIONE.

	//Adattamento della scala temporale di rappresentazione:
	if (fabs(tempo-t) > SOGLIA)  {
	  void far *prev_image;		// Si utilizza una finestra a scomparsa,
	  int x0=100, y0=100, x1=600, y1=240; // allora si deve salvare lo
	  unsigned size;		// sfondo sul quale la si disegna.
	  int tmp = imagesize(x0-1, y0-1, x1+1, y1+1);
	  my_win *awin;
		  Tunit=tempo/t*Tunit;  //Ricalcolo dell'unit di tempo.
	  if (tmp!=-1) {
	    char msg[50];
	    size = tmp;
	    if ((prev_image = (void far*)(malloc(size))) == NULL) {
	      closegraph();
	      printf("Error: not enough heap space in parameter().\n");
	      exit(1);
	    }//if.
	    getimage(x0-1, y0-1, x1+1, y1+1, prev_image);
	    simul_win->parq_win();
	    setfillstyle(SOLID_FILL,BLUE);
	    bar(1,1,640,16);
	    awin = new my_win(x0, y0, x1, y1, "AVVISO");
	    setcolor(YELLOW);
	    awin->textxy(10,10,"TEMPO DI SIMULAZIONE e TEMPO REALE non coincidono.");
	    setcolor(7);
	    awin->textxy(10,30," Si  effettuato un ADATTAMENTO della scala temporale");
	    awin->textxy(10,50,"al decimo di secondo:");
	    sprintf(msg,"NUOVA Tu = %f",Tunit);
	    awin->textxy(190,50,msg);
	    setcolor(YELLOW);
	    awin->textxy(100,90,"Premere un tasto per continuare...");
	    getch();
	    delete awin;
	    putimage(x0-1, y0-1, prev_image, COPY_PUT);
	    free((void*)prev_image);
	  }//if.
	}//if.

	//Distruzione degli oggetti ed abbandono della finestra di simulazione:
	delete Ks_1;
	delete Ks_2;
	delete Ms;
	delete Kp_1;
	delete Kp_2;
	delete Mp;
	simul_win->parq_win();
	break;
      }//END OF SIMULAZIONE.

      case F4: { //INIZIO RAPPRESENTAZIONI GRAFICHE:

	//Inizializzazione di variabili locali:
	float ks=(k1*k2)/(k1+k2); // Vedere sopra ...
	float kp=k1+k2;           // Vedere sopra ...
	float ws=sqrt(ks/m);      // Vedere sopra ...
	float wp=sqrt(kp/m);      // Vedere sopra ...
	float eps=mu/(2*m);       // Vedere sopra ...
	float alfas=sqrt(fabs(eps*eps-ws*ws)); // Vedere sopra ...
	float alfap=sqrt(fabs(eps*eps-wp*wp)); // Vedere sopra ...
	float fases=atan(eps/alfas);           // Vedere sopra ...
	float fasep=atan(eps/alfap);           // Vedere sopra ...
	float Cs=sqrt(fabs(ws*ws/(ws*ws-eps*eps))); // Vedere sopra ...
	float Cp=sqrt(fabs(wp*wp/(wp*wp-eps*eps))); // Vedere sopra ...
	float C1s=1.+eps/alfas;                     // Vedere sopra ...
	float C2s=1.-eps/alfas;                     // Vedere sopra ...
	float C1p=1.+eps/alfap;                     // Vedere sopra ...
	float C2p=1.-eps/alfap;                     // Vedere sopra ...
	float Ts=2*M_PI/ws;   	// Vedere sopra ...
	float Tp=2*M_PI/wp;     // Vedere sopra ...
	float As=m*G/ks;        // Vedere sopra ...
	float Ap=m*G/kp;        // Vedere sopra ...
	float A1=m*G/k1;        // Vedere sopra ...
	float A2=m*G/k2;        // Vedere sopra ...
	float div_num=100.;     // Numero divisioni dell'unit di tempo.
	float pix_for_unit=40.;//20.; // Vedere sopra ...
	float Tunit_num=14.;//15.; // Numero di unit di tempo che si graficheranno.
	float Dtmax_pix=Tunit_num*pix_for_unit;//Larghezza in pix. dell'asse
					       //temporale.
	float Dxmax_pix=5*pix_for_unit; // Come Dtmax ma per l'asse X.
	float dt=Tunit/div_num;  // Vedere sopra ...
	float t=0.;	      // Vedere sopra ...
	float kf1=k1/kp;      // Vedere sopra ...
	float kf2=k2/kp;      // Vedere sopra ...
	float Dl10st=kf2*Dl0s;// Vedere sopra ...
	float Dl20st=kf1*Dl0s;// Vedere sopra ...
	float Dtmax=Tunit_num*Tunit; // Intervallo temporale reale studiato.
	int num_iter=(int)(Dtmax/dt);// Numero di iterazioni nel ciclo di
				     // rappresentazione dei grafici.
	int i;  // Var. di ciclo.
	int ti; // Tempo discretizzato.
	int x0_pix=40;  // Posizione in pix. (ascissa) delle finestre dei grafici.
	int y0s_pix=270;//Come prima, Grafici spostamenti.
	int y0v_pix=270;//Come prima, Grafici fasi.
	int x0o_pix=150;			//Dati per la rappresentazione
	int y0o_pix=y0v_pix;			// tridimensionale.
	float cosfi=0.5, sinfi=sqrt(3.)/2.;	// idem...
	float vst,vpt;  //Velocit funzione del tempo, casi serie e parallelo.
	int vsi,vpi;	//Analoghi a vst e vpt ma nel discreto.
	// Vedere sopra ...
	float SW1s=0., SW2s=0., SW3s=0.;
	float SW1p=0., SW2p=0., SW3p=0.;
	char car;  //Carattere di scelta tra i grafici possibili
	int *orbitas, *orbitap; //Array di immagazzinamento tmp dei grafici
				// di fase nel tempo.

	//Scrittura dei dati calcolati sulla finestra dati:
	data_win->clear();
	gotoxy(60,4);printf("k1 = %6.3f [N/m]",k1);
	gotoxy(60,5);printf("k2 = %6.3f [N/m]",k2);
	gotoxy(60,6);printf("m  = %6.3f [kg]",m);
	gotoxy(60,7);printf("%c  = %6.3f [N*s/m]",230,mu);
	gotoxy(60,9);printf("Ts = %6.3f [s]",Ts);
	gotoxy(60,10);printf("Tp = %6.3f [s]",Tp);
	gotoxy(60,11);printf("As = %6.3f [m]",2.*fabs(As-Dl0s));
	gotoxy(60,12);printf("Ap = %6.3f [m]",2.*fabs(Ap-Dl0p));
	gotoxy(60,13);printf("X0s= %6.3f [m]",Dl0s);
	gotoxy(60,14);printf("X0p= %6.3f [m]",Dl0p);
	gotoxy(60,15);printf("Xsmin=%6.3f [m]",As-fabs(As-Dl0s));
	gotoxy(60,16);printf("Xsmax=%6.3f [m]",As+fabs(As-Dl0s));
	gotoxy(60,17);printf("Xpmin=%6.3f [m]",Ap-fabs(Ap-Dl0p));
	gotoxy(60,18);printf("Xpmax=%6.3f [m]",Ap+fabs(Ap-Dl0p));
	gotoxy(60,19);printf("Avs=%6.3f [m/s]",2.*fabs((Dl10st+Dl20st-A1-A2)*ws));
	gotoxy(60,20);printf("Avp=%6.3f [m/s]",2.*fabs((Dl0p-Ap)*wp));
	gotoxy(60,22);printf("Xu=%6.3f [m]",Xunit);
	gotoxy(60,23);printf("Vu=%6.3f [m/s]",Vunit);
	gotoxy(60,24);printf("Tu=%6.3f [s]",Dtmax/Dtmax_pix*pix_for_unit);

	//Preparazione della finestra grafici: assi ect...
	simul_win->parq_win();
	data_win->recall_win();
	simul_win->clear();
	setcolor(GREEN);
	simul_win->textxy(20,40,"ELABORAZIONE GRAFICA");
	setcolor(7);
	simul_win->textxy(20,60,"Nella finestra dati sono mostrati i risultati dei");
	simul_win->textxy(20,80,"calcoli sui dati impostati riguardo il sistema.");
	setcolor(YELLOW);
	simul_win->textxy(50,120,"GRAFICI POSSIBILI:");
	setcolor(WHITE);
	simul_win->textxy(50,140,"[1] Grafico delle x(t).");
	simul_win->textxy(50,160,"[2] Grafico delle v(t).");
	simul_win->textxy(50,180,"[3] Diagramma delle fasi.");
	simul_win->textxy(50,200,"[4] Diagramma delle fasi nel tempo.");
	setcolor(YELLOW);
	simul_win->textxy(40,220,"Premere il tasto corrispondente alla scelta...");
	setcolor(7);
	simul_win->textxy(20,260,"Se si preme un tasto diverso da [1], [2], [3] o [4]");
	simul_win->textxy(20,280,"si ritorna al men principale...");
	car = getch();
	switch (car) {
	case POSIZIONE : {
	  //Griglia per le posizioni:
	  delete simul_win;
	  simul_win = new my_win(1, 17, 639, 480, "MOLLE 1.3  -  SIMULAZIONE E GRAFICI");
	  setfillstyle(SOLID_FILL,24);bar(4,34,635,477);
	  outtextxy(x0_pix+20,y0s_pix-Dxmax_pix-20,"GRAFICO TEMPORALE DELLE POSIZIONI");
	  setcolor(7);
	  for(i=x0_pix; i<=x0_pix+Dtmax_pix; i+=(int)pix_for_unit)
	    line(i,y0s_pix-Dxmax_pix,i,y0s_pix+Dxmax_pix);
	  for(i=y0s_pix-Dxmax_pix; i<=y0s_pix+Dxmax_pix; i+=(int)pix_for_unit)
	    line(x0_pix,i,x0_pix+Dtmax_pix,i);
	  // Assi di riferimento posizione:
	  setcolor(WHITE);
	  line(x0_pix,y0s_pix,x0_pix+Dtmax_pix,y0s_pix);
	  line(x0_pix,y0s_pix-Dxmax_pix,x0_pix,y0s_pix+Dxmax_pix);
	  line(x0_pix-10,y0s_pix+Dxmax_pix-10,x0_pix,y0s_pix+Dxmax_pix);
	  line(x0_pix+10,y0s_pix+Dxmax_pix-10,x0_pix,y0s_pix+Dxmax_pix);
	  outtextxy(x0_pix+20,y0s_pix-Dxmax_pix+20,"x(t)");
	  outtextxy(x0_pix+Dtmax_pix-20,y0s_pix+20,"t");
	  // Unit di esempio:
	  setcolor(YELLOW);
	  line(x0_pix,y0s_pix-Dxmax_pix+40,x0_pix,y0s_pix-Dxmax_pix+40+(int)pix_for_unit);
	  line(x0_pix-5,y0s_pix-Dxmax_pix+40,x0_pix+5,y0s_pix-Dxmax_pix+40);
	  line(x0_pix-5,y0s_pix-Dxmax_pix+40+(int)pix_for_unit,x0_pix+5,y0s_pix-Dxmax_pix+40+(int)pix_for_unit);
	  outtextxy(x0_pix-20,y0s_pix-Dxmax_pix+40+(int)pix_for_unit/2,"Xu");
	  line(x0_pix+Dtmax_pix-80,y0s_pix,x0_pix+Dtmax_pix-80+(int)pix_for_unit,y0s_pix);
	  line(x0_pix+Dtmax_pix-80,y0s_pix+5,x0_pix+Dtmax_pix-80,y0s_pix-5);
	  line(x0_pix+Dtmax_pix-80+(int)pix_for_unit,y0s_pix+5,x0_pix+Dtmax_pix-80+(int)pix_for_unit,y0s_pix-5);
	  outtextxy(x0_pix+Dtmax_pix-80+(int)pix_for_unit/2,y0s_pix-20,"Tu");
	  break;
	}//POSIZIONI
	case VELOCITA : {
	  //Griglia delle velocit:
	  delete simul_win;
	  simul_win = new my_win(1, 17, 639, 480, "MOLLE 1.3  -  SIMULAZIONE E GRAFICI");
	  setfillstyle(SOLID_FILL,24);bar(4,34,635,477);
	  outtextxy(x0_pix+20,y0s_pix-Dxmax_pix-20,"GRAFICO TEMPORALE DELLE VELOCITA'");
	  setcolor(7);
	  for(i=x0_pix; i<=x0_pix+Dtmax_pix; i+=(int)pix_for_unit)
	    line(i,y0s_pix-Dxmax_pix,i,y0s_pix+Dxmax_pix);
	  for(i=y0s_pix-Dxmax_pix; i<=y0s_pix+Dxmax_pix; i+=(int)pix_for_unit)
	    line(x0_pix,i,x0_pix+Dtmax_pix,i);
	  // Assi di riferimento posizione:
	  setcolor(WHITE);
	  line(x0_pix,y0s_pix,x0_pix+Dtmax_pix,y0s_pix);
	  line(x0_pix,y0s_pix-Dxmax_pix,x0_pix,y0s_pix+Dxmax_pix);
	  line(x0_pix-10,y0s_pix+Dxmax_pix-10,x0_pix,y0s_pix+Dxmax_pix);
	  line(x0_pix+10,y0s_pix+Dxmax_pix-10,x0_pix,y0s_pix+Dxmax_pix);
	  outtextxy(x0_pix+20,y0s_pix-Dxmax_pix+20,"v(t)");
	  outtextxy(x0_pix+Dtmax_pix-20,y0s_pix+20,"t");
	  // Unit di esempio:
	  setcolor(YELLOW);
	  line(x0_pix,y0s_pix-Dxmax_pix+40,x0_pix,y0s_pix-Dxmax_pix+40+(int)pix_for_unit);
	  line(x0_pix-5,y0s_pix-Dxmax_pix+40,x0_pix+5,y0s_pix-Dxmax_pix+40);
	  line(x0_pix-5,y0s_pix-Dxmax_pix+40+(int)pix_for_unit,x0_pix+5,y0s_pix-Dxmax_pix+40+(int)pix_for_unit);
	  outtextxy(x0_pix-20,y0s_pix-Dxmax_pix+40+(int)pix_for_unit/2,"Vu");
	  line(x0_pix+Dtmax_pix-80,y0s_pix,x0_pix+Dtmax_pix-80+(int)pix_for_unit,y0s_pix);
	  line(x0_pix+Dtmax_pix-80,y0s_pix+5,x0_pix+Dtmax_pix-80,y0s_pix-5);
	  line(x0_pix+Dtmax_pix-80+(int)pix_for_unit,y0s_pix+5,x0_pix+Dtmax_pix-80+(int)pix_for_unit,y0s_pix-5);
	  outtextxy(x0_pix+Dtmax_pix-80+(int)pix_for_unit/2,y0s_pix-20,"Tu");
	  break;
	}//VELOCITA
	case FASE : {
	  //Griglia delle fasi:
	  delete simul_win;
	  simul_win = new my_win(1, 17, 639, 480, "MOLLE 1.3  -  SIMULAZIONE E GRAFICI");
	  setfillstyle(SOLID_FILL,24);bar(4,34,635,477);
	  setcolor(WHITE);
	  outtextxy(x0_pix+25,y0v_pix-Dxmax_pix-20,"DIAGRAMMA DELLE FASI DEL SISTEMA");
	  setcolor(7);
	  for(i=x0_pix; i<=x0_pix+2*Dxmax_pix; i+=(int)pix_for_unit)
	    line(i,y0v_pix-Dxmax_pix,i,y0v_pix+Dxmax_pix);
	  for(i=y0v_pix-Dxmax_pix; i<=y0v_pix+Dxmax_pix; i+=(int)pix_for_unit)
	    line(x0_pix,i,x0_pix+Dxmax_pix*2,i);
	  // Assi di riferimento:
	  setcolor(WHITE);
	  line(x0_pix,y0v_pix,x0_pix+Dxmax_pix*2,y0v_pix);
	  line(x0_pix+Dxmax_pix*2-10,y0v_pix-10,x0_pix+Dxmax_pix*2,y0v_pix);
	  line(x0_pix+Dxmax_pix*2-10,y0v_pix+10,x0_pix+Dxmax_pix*2,y0v_pix);
	  line(x0_pix+Dxmax_pix,y0v_pix-Dxmax_pix,x0_pix+Dxmax_pix,y0v_pix+Dxmax_pix);
	  line(x0_pix+Dxmax_pix-10,y0v_pix+Dxmax_pix-10,x0_pix+Dxmax_pix,y0v_pix+Dxmax_pix);
	  line(x0_pix+Dxmax_pix+10,y0v_pix+Dxmax_pix-10,x0_pix+Dxmax_pix,y0v_pix+Dxmax_pix);
	  outtextxy(x0_pix+20,y0v_pix-Dxmax_pix+20,"v(t)");
	  outtextxy(x0_pix+2*Dxmax_pix-20,y0v_pix+20,"x(t)");
	  //Unit di esempio:
	  setcolor(YELLOW);
	  line(x0_pix,y0v_pix-Dxmax_pix+40,x0_pix,y0v_pix-Dxmax_pix+40+(int)pix_for_unit);
	  line(x0_pix-5,y0v_pix-Dxmax_pix+40,x0_pix+5,y0v_pix-Dxmax_pix+40);
	  line(x0_pix-5,y0v_pix-Dxmax_pix+40+(int)pix_for_unit,x0_pix+5,y0v_pix-Dxmax_pix+40+(int)pix_for_unit);
	  outtextxy(x0_pix-20,y0v_pix-Dxmax_pix+40+(int)pix_for_unit/2,"Vu");
	  line(x0_pix+Dxmax_pix*2-80,y0v_pix,x0_pix+Dxmax_pix*2-80+(int)pix_for_unit,y0v_pix);
	  line(x0_pix+Dxmax_pix*2-80,y0v_pix-5,x0_pix+Dxmax_pix*2-80,y0v_pix+5);
	  line(x0_pix+Dxmax_pix*2-80+(int)pix_for_unit,y0v_pix-5,x0_pix+Dxmax_pix*2-80+(int)pix_for_unit,y0v_pix+5);
	  outtextxy(x0_pix+Dxmax_pix*2-80+(int)pix_for_unit/2,y0v_pix-20,"Xu");
	  break;
	}//FASI
	case ORBITA : { // Rappresentazione tridimensionale:
	  int Dxmax_pix1=Dxmax_pix/2;
	  int Dtmax_pix1=Dtmax_pix/2;
	  int x11=x0o_pix-Dxmax_pix1*sinfi, y11=y0o_pix+Dxmax_pix1*cosfi-Dxmax_pix1;
	  int x21=x11, y21=y0o_pix+Dxmax_pix1*cosfi;
	  int x31=x11, y31=y21+Dxmax_pix1;
	  int x12=x0o_pix+Dxmax_pix1*sinfi, y12=y0o_pix-Dxmax_pix1*cosfi-Dxmax_pix1;
	  int x22=x12, y22=y0o_pix-Dxmax_pix1*cosfi;
	  int x32=x12, y32=y22+Dxmax_pix1;
	  int x13=x12+Dtmax_pix1, y13=y12;
	  int x23=x13, y23=y22;
	  int x33=x13, y33=y32;
	  int x34=x33-Dxmax_pix1*sinfi, y34=y0o_pix+Dxmax_pix1;
	  int x35=x34-Dxmax_pix1*sinfi, y35=y31;
	  int poly[10];
	  poly[0]=x11; poly[1]=y11;
	  poly[2]=x31; poly[3]=y31;
	  poly[4]=x32; poly[5]=y32;
	  poly[6]=x12; poly[7]=y12;
	  poly[8]=poly[0]; poly[9]=poly[1];
	  delete simul_win;
	  simul_win = new my_win(1, 17, 639, 480, "MOLLE 1.3  -  SIMULAZIONE E GRAFICI");
	  setfillstyle(SOLID_FILL,24);bar(4,34,635,477);
	  setfillstyle(10,5);
	  bar(x12,y12,x33,y33);
	  fillpoly(5,poly);
	  setcolor(WHITE);
	  outtextxy(60,80,"DIAGRAMMA DELLE FASI NEL TEMPO");
	  line(x0o_pix,y0o_pix-Dxmax_pix1,x0o_pix,y0o_pix+Dxmax_pix1);
	  line(x21,y21,x22,y22);
	  line(x0o_pix,y0o_pix,x0o_pix+Dtmax_pix1,y0o_pix);
	  outtextxy(x0o_pix+Dtmax_pix1+10,y0o_pix,"t");
	  outtextxy(x0o_pix-40,y0o_pix-Dxmax_pix1-10,"v(t)");
	  outtextxy(x21-40,y21,"x(t)");
	  setcolor(7);
	  moveto(x31,y31);
	  lineto(x11,y11);lineto(x12,y12);lineto(x13,y13);lineto(x33,y33);
	  lineto(x32,y32);lineto(x31,y31);
	  line(x32,y32,x12,y12);
	  line(x22,y22,x23,y23);
	  line(x33,y33,x35,y35);
	  line(x35,y35,x31,y31);
	  line(x34,y34,x0o_pix,y34);
	  orbitas=(int*)malloc(2*num_iter*sizeof(int));
	  orbitap=(int*)malloc(2*num_iter*sizeof(int));
	}//ORBITA
	}//switch.

	if ( 	(car==POSIZIONE) ||
		(car==VELOCITA) ||
		(car==FASE) ||
		(car==ORBITA) ) {

	  //Determinazione del tipo di soluzione da usare:
	  if (eps==ws) SW2s=1.;
	  else if (eps>ws) SW3s=1.;
	     else SW1s=1.;
	  if (eps==wp) SW2p=1.;
	  else if (eps>wp) SW3p=1.;
	     else SW1p=1.;

	  for(i=1; i<=num_iter; i++) {// INIZIO LOOP DI DISEGNO DEI GRAFICI:

	  //Calcolo dell'allungamento all'istante t in [m]:
	    float E=exp(-eps*t);
	    Dl1st = A1 + A2 + (Dl10st+Dl20st-A1-A2)*
		( SW1s*E*Cs*cos(alfas*t-fases) +
		  SW2s*E*(1.+eps*t) +
		  SW3s*0.5*E*( C1s*exp(alfas*t) + C2s*exp(-alfas*t) ));
	    Dlpt  = Ap + (Dl0p-Ap)*
		( SW1p*E*Cp*cos(alfap*t-fasep) +
		  SW2p*E*(1.+eps*t) +
		  SW3p*0.5*E*( C1p*exp(alfap*t) + C2p*exp(-alfap*t) ));
	  //Calcolo delle velocit in [m/s] di oscillazione:
	    vst = (Dl10st+Dl20st-A1-A2)*
		( SW1s*Cs*E*( -eps*cos(alfas*t-fases) - alfas*sin(alfas*t-fases) )+
		  SW2s*E*eps*(-eps)*t+
		  SW3s*0.5*E*( C1s*(alfas-eps)*exp(alfas*t) - C2s*(alfas+eps)*exp(-alfas*t) ));
	    vpt = (Dl0p-Ap)*
		( SW1p*Cp*E*( -eps*cos(alfap*t-fasep) - alfap*sin(alfap*t-fasep) )+
		  SW2p*E*eps*(-eps)*t +
		  SW3p*0.5*E*( C1p*(alfap-eps)*exp(alfas*t) - C2p*(alfas+eps)*exp(-alfap*t) ));


	    //Calcolo dell'allungamento all'istante t in pixel:
	    Dl1si = (Dl1st/Xunit*pix_for_unit);
	    Dlpi  = (Dlpt/Xunit*pix_for_unit);
	    vsi = (vst/Vunit*pix_for_unit);
	    vpi = (vpt/Vunit*pix_for_unit);
	    ti = t/Dtmax*Dtmax_pix;

	    switch (car) {
	    case POSIZIONE : {
	      if (abs(Dl1si)<=Dxmax_pix) {
		setcolor(RED);setfillstyle(RED,SOLID_FILL);
		fillellipse(ti+x0_pix,Dl1si+y0s_pix,1,1);
	      }//if.
	      if (abs(Dlpi)<=Dxmax_pix) {
		setcolor(GREEN);setfillstyle(GREEN,SOLID_FILL);
		fillellipse(ti+x0_pix,Dlpi+y0s_pix,1,1);
	      }//if.
	      break;
	    }//POSIZIONE
	    case VELOCITA : {
	      if (abs(vsi)<=Dxmax_pix) {
		setcolor(RED);setfillstyle(RED,SOLID_FILL);
		fillellipse(ti+x0_pix,vsi+y0s_pix,1,1);
	      }//if
	      if (abs(vpi)<=Dxmax_pix) {
		setcolor(GREEN);setfillstyle(GREEN,SOLID_FILL);
		fillellipse(ti+x0_pix,vpi+y0s_pix,1,1);
	      }//if
	      break;
	    }//VELOCITA
	    case FASE : {
	      if ((abs(vsi)<=Dxmax_pix) && (abs(Dl1si)<Dxmax_pix)) {
		setcolor(RED);setfillstyle(RED,SOLID_FILL);
		fillellipse(Dl1si+x0_pix+Dxmax_pix,vsi+y0v_pix,1,1);
	      }//if.
	      if ((abs(vpi)<=Dxmax_pix) && (abs(Dlpi)<=Dxmax_pix)) {
		setcolor(GREEN);setfillstyle(GREEN,SOLID_FILL);
		fillellipse(Dlpi+x0_pix+Dxmax_pix,vpi+y0v_pix,1,1);
	      }//if.
	      break;
	    }//FASE
	    case ORBITA : { //Si disegnano solo le proiezioni:
	      orbitas[(i-1)*2]=0;
	      orbitas[i*2-1]=0;
	      if ( (abs(Dl1si)<=Dxmax_pix) && (abs(vsi)<=Dxmax_pix) ){
		orbitas[(i-1)*2]=(ti-Dl1si*sinfi)/2+x0o_pix;
		orbitas[i*2-1]=(-vsi+Dl1si*cosfi)/2+y0o_pix;
		putpixel((-Dl1si*sinfi)/2+x0o_pix,
			 (-vsi+Dl1si*cosfi)/2+y0o_pix,7);
		putpixel(ti/2+x0o_pix+Dxmax_pix/2*sinfi,
			-vsi/2+y0o_pix-Dxmax_pix/2*cosfi,7);
		putpixel((ti-Dl1si*sinfi)/2+x0o_pix,
			 Dl1si*cosfi/2+y0o_pix+Dxmax_pix/2,7);
	      }//if.
	      orbitap[(i-1)*2]=0;
	      orbitap[i*2-1]=0;
	      if ( (abs(Dlpi)<=Dxmax_pix) && (abs(vpi)<=Dxmax_pix) ){
		orbitap[(i-1)*2]=(ti-Dlpi*sinfi)/2+x0o_pix;
		orbitap[i*2-1]=(-vpi+Dlpi*cosfi)/2+y0o_pix;
		putpixel((-Dlpi*sinfi)/2+x0o_pix,
			 (-vpi+Dlpi*cosfi)/2+y0o_pix,7);
		putpixel(ti/2+x0o_pix+Dxmax_pix/2*sinfi,
			-vpi/2+y0o_pix-Dxmax_pix/2*cosfi,7);
		putpixel((ti-Dlpi*sinfi)/2+x0o_pix,
			 Dlpi*cosfi/2+y0o_pix+Dxmax_pix/2,7);
	      }//if.
	      break;
	    }//ORBITA
	    }//switch.

	    //Salto a istante successivo:
	    t += dt;

	  }//FINE LOOP DISEGNO GRAFICI.
	  if (car==ORBITA) { //Disegno delle orbite in tre dimensioni:
	    for(i=1; i<=num_iter; i++) {
	      if (orbitap[(i-1)*2]!=0) {
		setfillstyle(SOLID_FILL,GREEN);setcolor(GREEN);
		fillellipse(orbitap[(i-1)*2],orbitap[i*2-1],1,1);
	      }//if.
	      if (orbitas[(i-1)*2]!=0) {
		setfillstyle(SOLID_FILL,RED);setcolor(RED);
		fillellipse(orbitas[(i-1)*2],orbitas[i*2-1],1,1);
	      }//if.
	    }//for.
	    free((void*)orbitap);
	    free((void*)orbitas);
	  }//if.

	  //Legenda sulle linee:
	  setfillstyle(0,0);
	  bar(400,50,630,130);
	  setcolor(CYAN);
	  rectangle(400,50,630,130);
	  setcolor(RED);line(410,70,460,70);
	  outtextxy(470,70,"Molle in serie.");
	  setcolor(GREEN);line(410,90,460,90);
	  outtextxy(470,90,"Molle in parallelo");
	  setcolor(WHITE);line(410,110,460,110);
	  outtextxy(470,110,"Assi di riferimento.");

	  //Scrive sulla striscia men e ricostruisce le finestre:
	  setcolor(WHITE);
	  outtextxy(10,5,"Premere un tasto qualsiasi ...");
	  getch();
	  delete simul_win;
	  delete data_win;
	  simul_win = new my_win(1, 17, 450, 480, "MOLLE 1.3  -  SIMULAZIONE E GRAFICI");
	  data_win = new my_win(457, 17, 637, 480,"MOLLE 1.3 - DATI");
	}//if.

	data_win->clear();
	simul_win->clear();
	data_win->parq_win();
	simul_win->parq_win();
	break;
      }// FINE RAPPRESENTAZIONI GRAFICHE.

      case ESC: {//FINE ESECUZIONE:
	setfillstyle(BLUE,SOLID_FILL);
	bar(1,1,640,16);
	setcolor(WHITE);
	outtextxy(10,5,"Confermi uscita [s/n] ?");
	if (getch()=='s')
	  no_exit = FALSE;
	break;
      }//ESC

    }//End Of switch.
  } while (no_exit); //FINE DEL LOOP PRINCIPALE DEI COMANDI.

  //Eliminazione finestre di simulazione e dati:
  delete simul_win;
  delete data_win;

  //Uscita dalla grafica e fine programma:
  closegraph;
  restorecrtmode();
}//End Of Main Program.